                  /*--.          .-"-. 
                 /   ._.        /  ` \
                 \_  (__\       \_v_/
                 //   \\        //   \\
                ((     ))      ((     ))
 --""---""----""---""--
                  |||            |||                                               
   Author's note:  |              |                                                
                                                                                   
   This file is part of work done by Emil Persson, aka Humus.                      
   You're free to use the code in any way you like, modified, unmodified           
   or cut'n'pasted into your own work. But for the good of the whole               
   programming community I will ask you to as long as possible and to the          
   extent you find suitable to follow these points:                                
    * If you use this file and it's contents unmodified, please leave this note.   
    * If you use a major part of the file in your app, please credit the author(s) 
      in one way or the other as you find suitable, for instance a small notice    
      in the readme file may do, or perhaps a link to the authors site.            
    * Share your work and ideas too as much as you can.                            
     _______                                                                       
   /` _____ `\;,    Humus                                                          
  (__(^===^)__)';,  emiper-8@student.luth.se                         ___           
    /  :::  \   ,;  ICQ #47010716                                  /^   ^\         
   |   :::   | ,;'                                                (     )        
 '._______.'`--oOo--(_)--oOo--*/


#include "BumpPolygon.h"
#include "../Framework/OpenGL/glExtensions.h"

void BumpPolygon::draw(Renderer *renderer, const Vertex &pos, const Vertex &lPos, const float worldSize){
	if (plane.distance(lPos) < 0) return;

	renderer->setFragmentShader(0);
	renderer->setTextures(material->textures[0], material->textures[1], material->textures[2], 0);
	if (drawn){
		renderer->setBlending(GL_ONE, GL_ONE);
		glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 0, 0.0f,  0.0f,  0.0f,  0.0f);
	} else {
		glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, 0, 0.27f, 0.27f, 0.27f, 0.0f);
	}
	renderer->apply();
	
	glBegin(GL_TRIANGLE_FAN);
	for (unsigned int i = 0; i < nVertices; i++){
		Vertex v = vertices[i] - origin;
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB, v * s * scaleS, v * t * scaleT);

		v = lPos - vertices[i];
		glMultiTexCoord3fARB(GL_TEXTURE1_ARB, v * s, v * t, v * plane.normal);
		v = pos - vertices[i];
		glMultiTexCoord3fARB(GL_TEXTURE2_ARB, v * s, v * t, v * plane.normal);

		v = (lPos - vertices[i]) / worldSize;
		glMultiTexCoord3fvARB(GL_TEXTURE3_ARB, v);

		glVertex3fv(vertices[i]);
	}
	glEnd();

	drawn = true;
}

void BumpPolygon::drawDepth(Renderer *renderer, const Vertex &lPos, const float worldSize){
	renderer->setFragmentShader(1);
	renderer->apply();
	
	glBegin(GL_TRIANGLE_FAN);
	for (unsigned int i = 0; i < nVertices; i++){
		Vertex v = (lPos - vertices[i]) / worldSize;
		glTexCoord3fv(v);
		glVertex3fv(vertices[i]);
	}
	glEnd();
}

void BumpPolygon::drawAmbient(Renderer *renderer){
	if (drawn) return;

	renderer->setTextures(material->textures[0]);
	renderer->apply();

	glColor3f(0.27f, 0.27f, 0.27f);
	
	glBegin(GL_TRIANGLE_FAN);
	for (unsigned int i = 0; i < nVertices; i++){
		Vertex v = vertices[i] - origin;
		glTexCoord2f(v * s * scaleS, v * t * scaleT);
		glVertex3fv(vertices[i]);
	}
	glEnd();
}



class Polygon *BumpPolygon::create(const int size) const {
	return new BumpPolygon(size);
}









class Polygon *bumpPolyCreator(const int size){
	return new BumpPolygon(size);
}
